import os
import django

# Set up Django
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "orm_skeleton.settings")
django.setup()

# Import your models here
from django.db.models import Q, Count, Avg, F
from main_app.models import Movie, Director, Actor


# Create and run your queries within functions


def populate_db():
    # Creating Directors
    director1 = Director.objects.create(
        full_name="Francis Ford Coppola",
        birth_date="1939-04-07",
        nationality="Unknown",
        years_of_experience=50,
    )
    director2 = Director.objects.create(
        full_name="Akira Kurosawa",
        birth_date="1910-03-23",
        nationality="Unknown",
        years_of_experience=0,
    )
    director3 = Director.objects.create(
        full_name="Martin Scorsese",
        birth_date="1942-11-17",
        nationality="American and Italian",
        years_of_experience=60,
    )

    # Creating Actors
    actor1 = Actor.objects.create(
        full_name="Al Pacino",
        birth_date="1940-04-25",
        nationality="American",
        is_awarded=True,
    )
    actor2 = Actor.objects.create(
        full_name="Robert Duvall",
        birth_date="1931-01-05",
        nationality="American",
        is_awarded=False,
    )
    actor3 = Actor.objects.create(
        full_name="Joaquin Phoenix",
        birth_date="1974-10-28",
        nationality="American",
        is_awarded=True,
    )

    # Creating Movies
    movie1 = Movie.objects.create(
        title="The Godfather",
        release_date="1972-03-24",
        storyline="The aging patriarch of an organized crime dynasty transfers control to his reluctant son.",
        genre="Drama",
        rating=9.9,
        is_classic=True,
        is_awarded=True,
        director=director1,
        starring_actor=actor1,
    )
    movie1.actors.add(actor1, actor2)

    movie2 = Movie.objects.create(
        title="Apocalypse Now",
        release_date="1979-08-15",
        storyline="A journey upriver to confront a rogue colonel.",
        genre="Drama",
        rating=9.1,
        is_classic=False,
        is_awarded=False,
        director=director1,
        starring_actor=actor1,
    )
    movie2.actors.add(actor1, actor2)

    movie3 = Movie.objects.create(
        title="Seven Samurai",
        release_date="1954-04-26",
        storyline="Villagers hire samurai to protect them from bandits.",
        genre="Action",
        rating=8.6,
        is_classic=True,
        is_awarded=True,
        director=director2,
        starring_actor=actor3,
    )
    movie3.actors.add(actor3)


def get_directors(search_name=None, search_nationality=None):
    if search_name is None and search_nationality is None:
        return ""

    query = Q()
    query_name = Q(full_name__icontains=search_name)
    query_nationality = Q(nationality__icontains=search_nationality)

    if search_name is not None and search_nationality is not None:
        query |= query_name & query_nationality
    elif search_name is not None:
        query |= query_name
    else:
        query |= query_nationality

    directors = Director.objects.filter(query).order_by('full_name')

    if not directors:
        return ""

    result = []

    [result.append(f"Director: {director.full_name}, nationality: {director.nationality}, "
                   f"experience: {director.years_of_experience}") for director in directors]

    return '\n'.join(result)


def get_top_director():
    director = Director.objects.get_directors_by_movies_count().first()

    if not director:
        return ""

    return f"Top Director: {director.full_name}, movies: {director.num_movies}."


def get_top_actor():
    actor = Actor.objects.prefetch_related('starring_movies') \
        .annotate(
        num_of_movies=Count('starring_movies'),
        movies_avg_rating=Avg('starring_movies__rating')) \
        .order_by('-num_of_movies', 'full_name') \
        .first()

    if not actor or not actor.num_of_movies:
        return ""

    movies = ", ".join(movie.title for movie in actor.starring_movies.all() if movie)

    return f"Top Actor: {actor.full_name}, starring in movies: {movies}, " \
           f"movies average rating: {actor.movies_avg_rating:.1f}"


def get_actors_by_movies_count():
    actors = Actor.objects.annotate(num_movies=Count('actor_movies')) \
                 .order_by('-num_movies', 'full_name')[:3]

    if not actors or not actors[0].num_movies:
        return ""

    result = []
    for actor in actors:
        result.append(f"{actor.full_name}, participated in {actor.num_movies} movies")

    return '\n'.join(result)


def get_top_rated_awarded_movie():
    top_movie = Movie.objects\
        .select_related('starring_actor')\
        .prefetch_related('actors') \
        .filter(is_awarded=True) \
        .order_by('-rating', 'title') \
        .first()

    if top_movie is None:
        return ""

    starring_actor = top_movie.starring_actor.full_name if top_movie.starring_actor else "N/A"

    participating_actors = top_movie.actors.order_by('full_name').values_list('full_name', flat=True)
    cast = ", ".join(participating_actors)

    return f"Top rated awarded movie: {top_movie.title}, rating: {top_movie.rating:.1f}. " \
           f"Starring actor: {starring_actor}. Cast: {cast}."


def increase_rating():
    updated_movies = Movie.objects.filter(is_classic=True, rating__lt=10.0)

    if not updated_movies:
        return "No ratings increased."

    num_of_updated_movies = updated_movies.update(rating=F('rating') + 0.1)

    return f"Rating increased for {num_of_updated_movies} movies."

# populate_db()
# print(Director.objects.get_directors_by_movies_count())
# print(get_directors(search_name='', search_nationality=''))
# print(get_directors(search_name='Martin', search_nationality='Canadian'))
# print(get_top_director())
# print(get_top_actor())
# print(get_actors_by_movies_count())
# print(get_top_rated_awarded_movie())
# print(increase_rating())
